@@ -2,6 +2,7 @@
 
 import json
 from .storage import *
+from .tools.parallel import ParallelData
 
 class ModelCacheClass(object):
 
@@ -16,7 +17,7 @@ class ModelCacheClass(object):
 
         self.init__load_data(record)
 
-        assert self.item_id, "self.item_id should be assigned in self.load_data function!"
+        assert self.item_id is not None, "self.item_id should be assigned in self.load_data function!"
         assert isinstance(self.item_content, unicode), \
                 ("self.item_content should be assigned as unicode in self.load_data function! %s" \
                     % repr(self.item_content))
@@ -52,37 +53,20 @@ class ModelCacheClass(object):
     def pull_data(cls):
         print; print "[LOAD] %s [INTO] %s" % (cls.original.model.__module__, cls.__module__)
 
-        original_model_count = set_default_value([ \
-                lambda : cls.original.model.count(), \
-                lambda : len(cls.original.model), \
-                ], u"original.model is invalid!")
-
-        if len(cls) / float(original_model_count) < cls.original.percentage:
-            print "[load ids cache] ..."
-            cls.datadict.sync() # sync first
-            ids_cache = {item_id1 : True for item_id1, item1 in process_notifier(cls.datadict.datadict)}
-
-            items = []
-            for e1 in process_notifier(cls.original.model):
-                if cls.original.read_id_lambda(e1) in ids_cache: continue
-                if cls.original.filter_lambda(e1): continue
-
-                items.append(cls(e1))
-
-                if len(items) > 10000:
-                    cls.feed_data(items)
-                    items = []
-            cls.feed_data(items)
-            del ids_cache
+        # default is list, so mongodb, mysql can be compacted.
+        ParallelData.process(cls.original.model, 'list', cls.dbpath, \
+                output_lambda=lambda items: cls.feed_data(items), \
+                output_len_lambda=lambda : len(cls), \
+                id_func=cls.original.read_id_lambda,
+                )
 
     @classmethod
     def feed_data(cls, items=[]):
         # items 必定是list, 经过cPickle反序列化回来的
         """ 也许reopen在build_indexes解决sqlite close等问题 """
         for i1 in items:
+            if not isinstance(i1, cls): i1 = cls(i1)
             # Fix InterfaceError: Error binding parameter 0 - probably unsupported type
             assert isinstance(i1.item_id, (str, unicode, int,)), ("not " + repr(i1.item_id))
-
             cls.datadict[i1.item_id] = i1
-
         cls.datadict.sync()
